% The function deals with alignment of two adjacent sub-swathes if one of them is fully sea ice covers
% First, using the small water regions (windows with the size of 150 by 150 pixels) among sea ice to conduct power balancing;
% Second, if the two blocks of the two sides are both sea-ice covered, using the averaged balancing factor to balance the the full scene.

%--------calculate the minimum values in two sub-swaths:Sigma_SN-sigma_n_Mod-------------------
nyy1=max(firstRangeSample(count-1,:))+300:min(lastRangeSample(count-1,:))-300;
sigmaRaw_temp1=Sigma_SN(:,nyy1)-sigma_n_Mod(:,nyy1);
sigmaRaw_temp11=imresize(sigmaRaw_temp1,[rangx/150,length(nyy1)/150],'box');
sigmaRaw_temp11(isnan(sigmaRaw_temp11))=0.003;
ytemp(count-1) =min(min(sigmaRaw_temp11));

nyy2=max(firstRangeSample(count,:))+300:min(lastRangeSample(count,:))-300;
sigmaRaw_temp2=Sigma_SN(:,nyy2)-sigma_n_Mod(:,nyy2);
sigmaRaw_temp22=imresize(sigmaRaw_temp2,[rangx/150,length(nyy2)/150],'box');
sigmaRaw_temp22(isnan(sigmaRaw_temp22))=0.003;
ytemp(count) =min(min(sigmaRaw_temp22));
%--------calculate the minimum values in two sub-swaths: when Sigma_SN-sigma_n_Scaled----------------
ytemp2=zeros(1,5);
sigmaRaw_temp1=Sigma_SN(:,nyy1)-sigma_n_Scaled(:,nyy1);
sigmaRaw_temp11=imresize(sigmaRaw_temp1,[rangx/150,length(nyy1)/150],'box');
sigmaRaw_temp11(isnan(sigmaRaw_temp11))=0.003;
ytemp2(count-1) =min(min(sigmaRaw_temp11));

sigmaRaw_temp2=Sigma_SN(:,nyy2)-sigma_n_Scaled(:,nyy2);
sigmaRaw_temp22=imresize(sigmaRaw_temp2,[rangx/150,length(nyy2)/150],'box');
sigmaRaw_temp22(isnan(sigmaRaw_temp22))=0.004;
ytemp2(count) =min(min(sigmaRaw_temp22));
%----------------------------------------------------------------------------------
for ii=count-1:-1:1
    if ytemp_before==0 && ytemp2(ii)<0.0015 && ytemp2(ii)>0 %if the minimum is smaller than 0.0015, we think it's water.
        ytemp_before=ytemp(ii);  %ytemp_before==0, replys there is no standard level in the former sub-swath.
    end
end
% ytemp_before is the sigma level of the former sub-swath.
% The balance factor is the difference of this sub-swath's sigma level to the former one, in water regions.
if ytemp2(count)<0.0015 && ytemp_before~=0 
    kpb0(count)=ytemp(count)-ytemp_before; 
    firstAzimuthLineEW=firstAzimuthLine(count,:);
    firstAzimuthLineEW(find(isnan(firstAzimuthLineEW)))=[];
    for jj=1:length(firstAzimuthLineEW)
        nyy=firstRangeSample(count,jj):lastRangeSample(count,jj);
        nxx=firstAzimuthLine(count,jj):lastAzimuthLine(count,jj);
        sigma_n_Mod(nxx,nyy)=sigma_n_Scaled(nxx,nyy)+kpb0(count); 
    end
    
%%  boundary balance. Samely, using the blocks to calculate balancing factors.
else 
    times=20; mm=0;%set the iteration times.
    while 1   
        mm=mm+1;%m is the iteration number
        if mm>=times
            break;
        end
        nxx0=-199:0;kpb022=zeros(); %kpb022 is used to record the balance values in each two blocks
        [rangx,rangy]=size(Sigma_SN);
        for ii=1:ceil(rangx/200) 
            %------nxx is the azimuth width/lines of the block------------------------------------
            if ii*200<rangx
                nxx=nxx0+200*ii;
            else
                nxx=nxx(end)+1:rangx;
            end
            if length(nxx)<10
                break;
            end
            bound1l=5; bound2f=5;
            %------1)select the local regions---------------------------------------------------------------------
            sigmaRaw_temp1=Sigma_SN(nxx,min(lastRangeSample(count-1,:))-bound1l-20:min(lastRangeSample(count-1,:))-bound1l);
            sigmaRaw_temp2=Sigma_SN(nxx,max(firstRangeSample(count,:))+bound2f:max(firstRangeSample(count,:))+bound2f+20);

            sigma_n_Mod_temp1=sigma_n_Mod(nxx,min(lastRangeSample(count-1,:))-bound1l-20:min(lastRangeSample(count-1,:))-bound1l);
            sigma_n_Mod_temp2=sigma_n_Mod(nxx,max(firstRangeSample(count,:))+bound2f:max(firstRangeSample(count,:))+bound2f+20);

            ytemp1 =imresize(sigmaRaw_temp1,[length(nxx)/20,1],'box');  %calculate the box average, to pre-judge the ice-water conditions.
            ytemp2 =imresize(sigmaRaw_temp2,[length(nxx)/20,1],'box'); 

            %------2)if the difference of the maxima of two sides is too large--------------------------------------------
            if abs(max(max(ytemp2))-max(max(ytemp1)))>0.0045 ||isnan(max(max(ytemp2))) || isnan(max(max(ytemp1)))
                kpb022(ii)=nan;
            else
            %------3)calculate the difference of local regions in two sides-----------------------------------------------
                yy1=nanmean(nanmean(sigmaRaw_temp1-sigma_n_Mod_temp1)); %the mean values of local region.
                yy2=nanmean(nanmean(sigmaRaw_temp2-sigma_n_Mod_temp2)); %the mean values of local region.
                kpb022(ii)=yy2-yy1;  % the mean difference, i.e., the balance values.
                if abs(kpb022(ii))>=2*10^(-3) %if the mean difference is larger than 0.002 
                    kpb022(ii)=nan;
                end
            end
        end %end the "for"
        %------4)use the balance values in these homogeneous blocks to calculate the average balancing factor(kpb0(count))------------------- 
        kpb022(kpb022==0)=nan;
        if isnan(nanmean(kpb022))==0 %if there are homogeneous blocks with "not nan" values.
            kpb0(count)=kpb0(count)+nanmean(double(kpb022));
            disp(nanmean(kpb022));
        else
            if mm==1
                kpb0(count)=kpb0(count-1); 
            else
                kpb0(count)=0; 
                break;
            end
        end
        %------5)make alignment at the inter-swath---------------------------------------------------- 
        firstAzimuthLineEW=firstAzimuthLine(count,:);
        firstAzimuthLineEW(find(isnan(firstAzimuthLineEW)))=[];
        for ii=1:length(firstAzimuthLineEW) %use kpb0 in each scene in each sub-swath
            nyy=firstRangeSample(count,ii):lastRangeSample(count,ii);
            nxx=firstAzimuthLine(count,ii):lastAzimuthLine(count,ii);
            sigma_n_Mod(nxx,nyy)=sigma_n_Scaled(nxx,nyy)+kpb0(count); 
        end
        %------6)if the balance values is smaller than 0.000001,stop the iteration-----------
        if abs(nanmean(kpb022))<0.01*10^(-4)
            break;
        end
    end %while
end 