%% for the noise scaling, image blocks are used here.
% 1)divide each sub-swath into number of blocks, and calculate the scaling factor and variance factor in each block.
% 2)use the variance factor to judge the the blocks are homogeneous or inhomogeneous.
% 3)use the homogeneous blocks to calculate the mean scaling factor for each sub-swath.

% For details of explanation, please refer to Section III, Part A "Determination of the optimal noise scaling factor" in the paper by Sun and Li (2020)

%% the calibrated values
Sigma_SN=(DN_masked.^2)./cali.^2;
Sigma_NESZ=rsNoise./cali.^2; 
Sigma_N_ESA=gggDS.*rsNoise./cali.^2; 
    
%% 1) calculate the optimal scaling factor & variance factor in segmented blocks in each sub-swath
% The length of each block is 200 lines in azimuth direction
% Calculate the optimal scaling factor kns(count,ii) in each block, where "count" is the number of sub-swath,"ii" is the number of blocks.

sWnum=length(firstAzimuthLine(:,1)); %the number of sub-swath. 
[rangx,rangy]=size(DN_masked);
kns=NaN(sWnum,ceil(rangx/200.0)); %define the scaling factor array "kns"
varss=NaN(sWnum,ceil(rangx/200.0)); %define the variance array "varss"
nxx0=-199:0; 
for count=1:sWnum 
    for ii=1:ceil(rangx/200.0) %get optimal scale parameter: kns(count,ii) in each block in each swath 
        %------nxx is the azimuth width/lines of the block--------------------------------------------
        if ii*200<rangx
            nxx=nxx0+200*ii; 
        else
            nxx=200*(ii-1)+1:rangx;
        end
        if length(nxx)<10 
            break;
        end
        %------nyy is the range width/pixels of the block(i.e.,of the sub-swath)---------------------- 
        if count==1  
            nyy=max(firstRangeSample(count,:))+100:min(lastRangeSample(count,:))-10;
			%In EW1, there is a jump in the noise vector in the fore-boundary.
        elseif count==5
            nyy=max(firstRangeSample(count,:))+10:min(lastRangeSample(count,:))-100;
			%In EW5, there is a jump in the noise vector in the back-boundary.
        else                                                                   
            nyy=max(firstRangeSample(count,:))+10:min(lastRangeSample(count,:))-10;
			%abandon the narrow boundary
        end
        length_range=length(nyy);
        %---------------------------------------------------------------------------------------------
        clear('sigmaRaw','sigma0nnn','rsNoise11');
        sigmaRaw=nanmean(Sigma_SN(nxx,nyy)); % the range profile of undenoised/raw sigma values
        %-------------remove the land,i.e.,the value "nan"-------------------------------------
        temp1=find(isnan(sigmaRaw));
        if length(temp1)>1 
            temp2=zeros;mm2=2;
            for mm=1:length(temp1)-1
                if temp1(mm+1)-temp1(mm)>1
                    temp2(mm2)=temp1(mm);
                    mm2=mm2+1;
                    temp2(mm2)=temp1(mm+1);
                    mm2=mm2+1;
                end
            end
            temp2(1)= temp1(1); temp2(mm2)= temp1(end); nyytemp=1;
            for mm=1:2:length(temp2)-1
                if temp2(mm)>150  
                    nyytemp=[nyytemp,temp2(mm)-149:temp2(mm+1)]; %avoid the high values(wide around 150 pixels) in the land boundaries
                else
                    nyytemp=[nyytemp,temp2(mm):temp2(mm+1)];
                end
                if temp2(mm+1)<length(nyy)-150
                    nyytemp=[nyytemp,temp2(mm+1)+1:temp2(mm+1)+149];
                end
            end
            nyytemp(1)=[];
            sigmaRaw(nyytemp)=[]; %the range profile of undenoised sigma values
            nyy(nyytemp)=[];
        end
        if length(nyy)<0.8*length_range
            continue;
        end
        %---------------------------------------------------------------------------------------
        %------calculate the scaling factor(Kns(count,ii))--------------------------------------
        sigma0nnn=nanmean(gggDS(nxx,nyy).*Sigma_NESZ(nxx,nyy)); %the range profile of unmodified ESA-provided noise
        rsNoise11=nanmean(rsNoise(nxx,nyy)); %the range profile of range noise vector
        wi=gradient(rsNoise11); %the gradient

        mm=1;rss=zeros(); 
        clear sum;
        for k=0.8:0.01:2  %set the selection of "k".Based on the experience, we change the range of k to be 0.8~2.0.
            sk=sigmaRaw-k.*sigma0nnn; 
            p=polyfit(nyy,sk,1);%linear fit of sk
            skfit=polyval(p,nyy);%linear fit values
            rss(mm)=double(sum(abs(wi).*(skfit-sk).^2));
            mm=mm+1;
        end
        [M,I] = nanmin(rss); %when "rss" has the minimum,
        kns(count,ii)=0.8+(I-1)*0.01; %the "k" is the optimal scaling factor.
        knstemp=kns(count,ii);
        %---------------------------------------------------------------------------------------------
        %--------1)calculate the variance factor(varss(count,ii))-------------------------------------- 
        clear('sigmaRaw','sigma0nnn');
        sigmaRaw=mean(Sigma_SN(nxx,nyy)); %the range profile of undenoised sigma values
        sigma0nnn=mean(gggDS(nxx,nyy).*Sigma_NESZ(nxx,nyy));%the range profile of unmodified ESA-provided noise
        meanline=knstemp.*sigma0nnn+mean(mean(sigmaRaw-knstemp.*sigma0nnn));%the equivalent mean line of the raw sigma
        deviation=sigmaRaw-meanline;%the deviation
        varss3(count,ii)=var(deviation,1);
        
        p=polyfit(nyy,deviation,1);%linear fit of deviation
        deviatfit=polyval(p,nyy);
        varss(count,ii)=double(sum((deviation-deviatfit).^2))/length(nyy); 
        %------------show the profiles, which can help to make the judgment------------------------
%         figure;plot(nyy,sigmaRaw,'color',[0.45 0.45 0.45],'linewidth',0.7);hold on;
%         plot(nyy,knstemp.*sigma0nnn,'-.','color',[0.3 0.3 0.3],'linewidth',0.7);
%         plot(nyy,meanline,'k','linewidth',0.7)%'color',[0.5 0.5 0.5]);
%         plot(nyy,deviation,'color',[0.65 0.65 0.65],'linewidth',0.7);
%         xlabel('range pixel index');
%         ylabel('sigma nought values');
%         legend('Raw sigma nought values','Scaled noise sigma nought values','Equivalent mean values','Deviation');
%         text(max(firstRangeSample(count,:))+300,0.0006,['kns(1,48)=',num2str(knstemp)]);
%         text(max(firstRangeSample(count,:))+300,0.0009,['var(48)=',num2str(varss(count,ii))]);
    end
end
clear('sigmaRaw','sigma0nnn','rsNoise11','meanline','deviation','deviatfit');
%---------------------------------------------------------------------------------------------------- 
%If one would like to check the variance factors among different types of surface, the following part can be used to present the corresponding histogram.  

% Show the probability histogram of the variance factor(varss) 
% clear h;
% maxxx=max(max(varss));
% minnn=min(min(varss));
% rssData = varss(:)'; 
% rssData(isnan(rssData))=[];
% nbins=300;
% figure;h = histogram(rssData,nbins);
% a=h.Values;
% sumc1=length(find(isnan(varss)==0));
% a=a./sumc1;%the probability histogram
% aa=cumsum(a);%the cumulative probability histogram
% bW=h.BinWidth;
% plot(a);title('\fontsize{16}the probability histogram');
% xvalue=linspace(minnn,maxxx,10);  
% set(gca,'XTickMode','manual');set(gca,'XTickLabelMode','manual');
% set(gca,'xtick',linspace(1,300,10));set(gca,'xticklabel',xvalue);
% set(gcf,'unit','normalized','position',[0.2,0.2,0.64,0.38]);
% set(gcf, 'PaperPositionMode', 'manual');set(gcf, 'PaperUnits', 'points');
% set(gcf, 'PaperPosition', [0 0 680 280]); 
% close all
%---------------------------------------------------------------------------------------------------- 

%% 2) Use the variance factor(varss) to simply distinct the "all land" and "all ice" images.
maxxx=max(max(varss));
minnn=min(min(varss));
imgStyle=1;%ice-water coexisting image
if length(find(isnan(varss)))>=0.95*length(varss)*sWnum
    imgStyle=2;
    disp('almost all land');  
elseif ( minnn >2*10^(-8) && maxxx>2*10^(-6))
    imgStyle=-2;
    disp('large area of ice');
end
disp(imgStyle);
%--------if it's almost all land, simply using the empirical values--------------------------
if imgStyle==2 
    kns2=[1.1,1.0,1.0,1.0,1.0]; 
    Meanbalance; %refer to the function: meanbalance.m
end

%% 3) Use the variance factor(varss) to select the homogeneous blocks, then to calculate the mean vlues of kns(count,ii) 
if imgStyle~=2   
    kns12=kns; 
    for count=1:sWnum 
        index1=[find(varss(count,:)>8*10^(-8)),find(isnan(varss(count,:)))]; %if the varss is larger than the threhold(8*10^(-8)), the block is regarded as inhomogeneous block.
        if isnan(index1)==0
            kns12(count,index1)=nan; %abandon the varss in the inhomogeneous blocks.
        end
    end
    kns2=nanmean(kns12,2); %acquire the average values of the scaling factor kns in homogeneous blocks.

    for count=1:sWnum    %if there is no homogeneous blocks in one sub-swath
        if isnan(kns2(count)) && count==1
            kns2(count)=1.1;  %use the average values of the scaling factor kns in all blocks.
        elseif isnan(kns2(count)) %if it is all land in this sub-swath,just no scaling. 
            kns2(count)=1.0;
        end
    end
end
kns2(5)=1.0;  %new setting: 20220905

if imgStyle==-2  %new setting: 20220905
    kns2(2:end) = 1.0; 
end
